function [AtA,Y,numIter,tElapsed,finalResidual]=KSRDL(XtX,k,option)
% Kernel dictionary learning using l1LS X=AY given X
% Yifeng Li
% Feb. 10, 2012

tStart=tic;
optionDefault.lambda=0.1;
optionDefault.SRMethod='nnls';
optionDefault.kernel='rbf';
optionDefault.param=[];
optionDefault.iter=200;
optionDefault.dis=1;
optionDefault.residual=1e-4;
optionDefault.tof=1e-4;
if nargin<3
    option=optionDefault;
else
    option=mergeOption(option,optionDefault);
end

c=size(XtX,1);

% Initialize AtA
% [U,~,~]=svd(X,'econ');
% A=U(:,1:k);
% AtA=computeKernelMatrix(A,A,option);
% AtX=computeKernelMatrix(A,X,option);
% XtX=computeKernelMatrix(X,X,option);
Y=rand(k,c);
if c==k
    AtA=Y'\XtX/Y;
    AtX=Y'\XtX;
else
    pinvY=pinv(Y);
    AtA=pinvY'*XtX*pinvY;
    AtX=pinvY'*XtX;
end

prevRes=Inf;
% alternative updating
for i=1:option.iter
    %     % normalize AtA and AtX
    %     [AtA,~,XtX,~,AtX]=normalizeKernelMatrix(AtA,XtX,AtX);
    
    % update Y
    [zeroA,tf0]=iszero(sum(AtA));
    AtA(zeroA,:)=[];
    AtA(:,zeroA)=[];
    AtX(zeroA,:)=[];
    if tf0 && k>1
        k=k-1;
    end
    if k<1
       error('k<1, impossible!'); 
    end
    switch option.SRMethod
        case 'l1ls'
            Y=l1LSKernelBatchDL(AtA,AtX,diag(XtX),option);
        case 'l1qpAs'
            Y=l1QP(AtA,-AtX,option.lambda);
        case 'nnls'
            Y=NNLSKernelBatchDL(AtA,AtX,diag(XtX),option);
        case 'l1nnls'
            Y=l1NNLSKernelBatchDL(AtA,AtX,diag(XtX),option);
    end
    % update kernel matrices
    if c==k
        AtA=Y'\XtX/Y;
        AtX=Y'\XtX;
    else
        pinvY=pinv(Y);
        AtA=pinvY'*XtX*pinvY;
        AtX=pinvY'*XtX;
    end
    % normalize AtA and AtX
    [AtA,~,XtX,~,AtX]=normalizeKernelMatrix(AtA,XtX,AtX);
    
    if mod(i,20)==0 || i==option.iter
        if option.dis
            disp(['Iterating >>>>>> ', num2str(i),'th']);
        end
        switch option.SRMethod
            case 'l1ls'
                curRes=trace(XtX-Y'*AtX-AtX'*Y+Y'*AtA*Y)+option.lambda*sum(sum(abs(Y)));
            case 'l1qpAs'
                curRes=trace(XtX-Y'*AtX-AtX'*Y+Y'*AtA*Y)+option.lambda*sum(sum(abs(Y)));
            case 'nnls'
                curRes=trace(XtX-Y'*AtX-AtX'*Y+Y'*AtA*Y);
            case 'l1nnls'
                curRes=trace(XtX-Y'*AtX-AtX'*Y+Y'*AtA*Y);
        end
        fitRes=prevRes-curRes;
        prevRes=curRes;
        if option.tof>=fitRes || option.residual>=curRes || i==option.iter
            disp(['DL successes!, # of iterations is ',num2str(i),'. The final residual is ',num2str(curRes)]);
            numIter=i;
            finalResidual=curRes;
            break;
        end
    end
end
tElapsed=toc(tStart);
end