function [Di,Ei,newX] = updateDi(Y,X,index,classn, Fish_ipts,Fish_par,alpha, beta, lambda)
%This routione solves the following optimization problem,
%min |D|_*+beta*|E|_2,1+lambda*r(D)
%s.t., Y = DX+E
% Inputs: (1)A: the training data
%         (2)X: the coefficient matrix of the training data
%         (3)index: the label of the class being preocessed
%         (4)classn: the total number of the classes
%         (5)Fish_ipts
%                     .D the dictionary in the last iteration
%                     .trls the labels of training data
%         (6)Fish_par
%                     .dls the labels of the dictionary atoms
%        

D = Fish_ipts.D;
trls = Fish_ipts.trls;
dls = Fish_par.dls;

Do = D(:,dls~=index);
Di = D(:,dls==index);
Yi = Y(:,trls==index);
Yo = Y(:,trls~=index);
Xi = X(:,trls==index);
Xo = X(:,trls~=index);

Xi_i = Xi(dls==index,:);
Xi_o = Xi(dls~=index,:);
Xo_i = Xo(dls==index,:);
Xo_o = Xo(dls~=index,:);




tol = 1e-6;
%maxIter = 1e4;
maxIter=120;
[d n] = size(Yi);
m = size(Di,2);
rho = 1.1;
max_mu = 1e30;
mu = 1e-3;


V = zeros(m,m);
for t_i = 1:classn
    if t_i~=index
        Xt_i = X(dls==index,trls==t_i);
        V = V + Xt_i*Xt_i';
    end
end
V = 2*(lambda/mu)*V;

%% Initializing optimization variables
J = zeros(d,m);
Ei = sparse(d,n);

T1 = zeros(d,n);
T2 = zeros(d,m);
T3 = zeros(m,n);

%% Start main loop
iter = 0;
disp(['initial,rank=' num2str(rank(Di))]);
while iter<maxIter
    iter = iter +1;
    %update Z
    temp = Xi_i + T3/mu;
    Z = max(0,temp - 1/mu) + min(0,temp+1/mu);
    
    %update Xi_i
    Xi_i = inv(Di'*Di + eye(m))*(Di'*Yi-Di'*Ei+Z+(Di'*T1-T3)/mu);
    
    %update J
    temp = Di + T2/mu;
    [U,sigma,VV] = svd(temp,'econ');
    sigma = diag(sigma);
    svp = length(find(sigma>alpha/mu));
    if svp>=1
        sigma = sigma(1:svp)-alpha/mu;
    else
        svp=1;
        sigma=0;
    end
    J = U(:,1:svp)*diag(sigma)*VV(:,1:svp)';
    if norm(J)~=0
        for j = 1:size(J,2)
            J(:,j) = J(:,j)./norm(J(:,j));
        end
    end
    
    %update Di
%     part1 = 2*(lambda/mu)*(Yi*Xi_i'+Do*Xi_o*Xi_i');
%     part2 = Yi*Xi_i'-Ei*Xi_i'+J+(T1*Xi_i'-T2)/mu;
%     part3 = 2*(lambda/mu +1)*(Xi_i*Xi_i')+eye(m);
%     Di = (part1+part2)/(part3); %for faster inv function

    Di = (J+Yi*Xi_i' - Ei*Xi_i'+(T1*Xi_i'-T2)/mu)*inv(eye(m)+Xi_i*Xi_i'+V);
    for j = 1:size(Di,2)
        Di(:,j) = Di(:,j)./norm(Di(:,j));
    end
    
    
    
    %update Ei
    temp = Yi - Di*Xi_i+T1/mu;
    %Ei = solve_l1l2(temp,beta/mu);
    Ei = max(0,temp-beta/mu)+min(0,temp+beta/mu);
    
    leq1 = Yi - Di*Xi_i - Ei;
    leq2 = Di - J;
    leq3 = Xi_i-Z;
    stopC = max([max(max(abs(leq1))),max(max(abs(leq2))),max(max(abs(leq3)))]);
    if iter==1 || mod(iter,50)==0 || stopC<tol
        disp(['iter ' num2str(iter) ',mu=' num2str(mu,'%2.1e') ...
            ',rank=' num2str(rank(Di,1e-3*norm(Di,2))) ',stopALM=' num2str(stopC,'%2.3e')]);
    end
    if stopC<tol 
        disp('Update Di done.');
        break;
    else
        T1 = T1 + mu*leq1;
        T2 = T2 + mu*leq2;
        T3 = T3 + mu*leq3;
        mu = min(max_mu,mu*rho);
    end
end
newX = X;
Xi(dls==index,:) = Xi_i;
newX(:,trls==index) = Xi;


function [E] = solve_l1l2(W,lambda)
n = size(W,2);
E = W;
for i = 1:n
    E(:,i) = solve_l2(W(:,i),lambda);
end

function [x] = solve_l2(w,lambda)
nw = norm(w);
if nw>lambda
    x=(nw-lambda)*w/nw;
else
    x = zeros(length(w),1);
end
